#include "DiagDataSendingWorker.h"

static quint64 port = 8040;

DiagDataSendingWorker::DiagDataSendingWorker(QTcpServer *parent)
    : QTcpServer{parent}
{
    s_timer_restart = new QTimer(this);
    connect(s_timer_restart, &QTimer::timeout,
            this, &DiagDataSendingWorker::startServer,
            Qt::QueuedConnection);

    startServer();
    f_sendingDataToServer = QSettings().value("send_data_flag", true).toBool();

    m_urls = QStringList()<<"https://telemetry.cellframe.net/diag_report"<< "https://engine-minkowski.kelvpn.com/diag_report";
    m_netMngr = new QNetworkAccessManager(this);
    connect(m_netMngr, &QNetworkAccessManager::finished, this, [=](QNetworkReply*r) {
        qDebug() <<"Data sent. " << r->error();
    });
}

void DiagDataSendingWorker::startServer()
{
    if( !listen(QHostAddress::Any, port) )
    {
        s_timer_restart->stop();
        s_timer_restart->start(5000);

        qWarning()<<"Unable start diagtool server" << errorString();
    }
    else{
        connect(this, &QTcpServer::newConnection,
                this, &DiagDataSendingWorker::onNewConnection,
                Qt::QueuedConnection);

        serverStatus = true;
        qInfo()<<"Diagtool server is started";
    }
}

void DiagDataSendingWorker::onNewConnection()
{
    qDebug()<<"New connection...";
    QTcpSocket * _socket = nextPendingConnection();

    if(_socket->state() != QAbstractSocket::ConnectedState)
        return;

    s_tcpSocketList.push_back(_socket);

    connect(_socket, &QTcpSocket::disconnected,
            this, &DiagDataSendingWorker::onClientDisconnect,
            Qt::QueuedConnection);

    connect(_socket, &QTcpSocket::readyRead,
          this, &DiagDataSendingWorker::onReadyRead,
          Qt::QueuedConnection);


    qDebug()<<"Client's count:" << s_tcpSocketList.length();
}

void DiagDataSendingWorker::onClientDisconnect()
{
    qDebug()<<"Client disconnected";
    QTcpSocket* _socket = dynamic_cast<QTcpSocket*>(sender());
    s_tcpSocketList.removeOne(_socket);
    qDebug()<<"Client's count:" << s_tcpSocketList.length();
}

void DiagDataSendingWorker::onReadyRead()
{
    QTcpSocket* _socket = dynamic_cast<QTcpSocket*>(sender());
    if ( _socket == nullptr ||
         _socket->state() != QAbstractSocket::ConnectedState) { return; }

    QJsonParseError error;

    QByteArray rcvData = _socket->readAll();
    QJsonDocument encodedJson = QJsonDocument::fromJson(rcvData, &error);

    if(error.error != QJsonParseError::NoError)
        return;

    QJsonObject rcvObj = encodedJson.object();
    if(!rcvObj["send_data_flag"].isNull() && rcvObj["send_data_flag"].isBool())
    {
        f_sendingDataToServer = rcvObj["send_data_flag"].toBool();
        QSettings().setValue("send_data_flag",f_sendingDataToServer);
    }
    else
        return;
}

void DiagDataSendingWorker::sendData(QJsonDocument diagdata, QString mac)
{
    if(serverStatus && s_tcpSocketList.length())
    {
        for(auto client: qAsConst(s_tcpSocketList))
            client->write(diagdata.toJson());
    }

    if(!f_sendingDataToServer)
        return;

    for (const auto &url_s : qAsConst(m_urls))
    {
        QUrl url = QUrl(url_s);

        auto req = QNetworkRequest(url);
        req.setHeader(QNetworkRequest::ContentTypeHeader,"application/x-www-form-urlencoded");
        QJsonObject obj = diagdata.object();
        obj.insert("mac",mac);
        diagdata.setObject(obj);
        m_netMngr->post(req, diagdata.toJson());
        qDebug()<<"Post request by url: " << url.toString();
    }
}
