#include "DiagnosticWorker.h"


static QString group = "global.users.statistic";

DiagnosticWorker::DiagnosticWorker(QObject * parent)
    : QObject{parent}
{

}

DiagnosticWorker::~DiagnosticWorker()
{
    delete s_uptime_timer;
    delete s_elapsed_timer;
    delete m_diagnostic;
}

void DiagnosticWorker::init()
{
    s_diagDataWorker = new DiagDataSendingWorker();

    s_uptime_timer = new QTimer(this);
    connect(s_uptime_timer, &QTimer::timeout,
            this, &DiagnosticWorker::slot_uptime,
            Qt::QueuedConnection);
    s_uptime_timer->start(1000);

    s_elapsed_timer = new QElapsedTimer();
    s_elapsed_timer->start();

#ifdef Q_OS_LINUX
    m_diagnostic = new LinuxDiagnostic();
#elif defined Q_OS_WIN
    m_diagnostic = new WinDiagnostic();
#elif defined Q_OS_MAC
    m_diagnostic = new MacDiagnostic();
#endif

    m_notifyWorker = new NotifyWorker();
    m_notifyWorker->rcvNetObj(m_diagnostic->roles_processing());


    connect(m_diagnostic, &AbstractDiagnostic::data_updated,
            this, &DiagnosticWorker::slot_diagnostic_data,
            Qt::QueuedConnection);

    m_diagnostic->start_diagnostic();
    m_diagnostic->set_timeout(3000); //30 sec

    m_isSendData = QSettings().value("diag_send_data", "1").toInt();

    m_initStatus = true;

    emit sigInitStatusChanged(m_initStatus);
}

void DiagnosticWorker::switchSendData()
{
    m_isSendData = !m_isSendData;

    if(!m_isSendData)
        m_diagnostic->stop_diagnostic();
    else
        m_diagnostic->start_diagnostic();

    QSettings().setValue("diag_send_data", m_isSendData);

    emit sigSendDataStatusChanged(m_isSendData);
}

void DiagnosticWorker::openConfigFile()
{
    QDesktopServices::openUrl(m_diagnostic->s_jsonFilePath);
}

void DiagnosticWorker::getSendedData()
{
    QFile sndData(m_diagnostic->s_sendedDataFilePath);
    sndData.open(QIODevice::WriteOnly);
    sndData.write(m_lastSendedPack.toJson());

    QDesktopServices::openUrl(m_diagnostic->s_sendedDataFilePath);
}

DiagnosticWorker &DiagnosticWorker::getInstance()
{
    static DiagnosticWorker instance;
    return instance;
}

void DiagnosticWorker::slot_diagnostic_data(QJsonDocument data)
{
    //insert uptime dashboard and more into system info
    QJsonObject obj = data.object();
    QJsonObject system = data["system"].toObject();
    QJsonObject proc = data["process"].toObject();

    system.insert("uptime_diagtool", s_uptime);
    system.insert("time_update_unix", QDateTime::currentSecsSinceEpoch());

//    system.insert("time_update", QDateTime::currentDateTime().toString("dd.MM.yyyy hh:mm"));
    obj.insert("system",system);

    if(proc["status"].toString() == "Offline") //if node offline - clear version
        m_node_version = "";
    else
    if(m_node_version.isEmpty())
    {
        QProcess proc;
        proc.setProgram(QString(CLI_PATH));
        proc.setArguments(QStringList()<<"version");
        proc.start();
        if(proc.waitForFinished(5000));
        {
            QString result = proc.readAllStandardOutput();
            if(result.contains("version"))
            {
                result = result.split("version")[1];
                m_node_version = result.split('\n', QString::SkipEmptyParts).first().trimmed();
            }
        }


        m_notifyWorker->nodeIsOnline();
    }

    proc.insert("version", m_node_version);
    proc.insert("node_init", m_notifyWorker->m_docNodeInit.object());
    obj.insert("process",proc);
    data.setObject(obj);
    m_lastSendedPack = data;

    s_diagDataWorker->sendData(data, m_diagnostic->s_mac.toString());
}

void DiagnosticWorker::slot_uptime()
{
    s_uptime = m_diagnostic->get_uptime_string(s_elapsed_timer->elapsed()/1000);
}
