#ifndef DIAGNOSTICWORKER_H
#define DIAGNOSTICWORKER_H

#include <QObject>
#include <QTimer>
#include <QElapsedTimer>
#include <QSettings>
#include <QThread>
#include <QDesktopServices>
#include <QSettings>

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "NotifyWorker.h"
#include "DiagDataSendingWorker.h"

#ifdef Q_OS_LINUX
    #include "LinuxDiagnostic.h"
#elif defined Q_OS_WIN
    #include "WinDiagnostic.h"
#elif defined Q_OS_MAC
    #include "MacDiagnostic.h"
#endif


class DiagnosticWorker : public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(DiagnosticWorker)

    explicit DiagnosticWorker(QObject *parent = nullptr);
    ~DiagnosticWorker();

public:
    static DiagnosticWorker &getInstance();

    void init();

    Q_INVOKABLE bool getInitStatus(){return m_initStatus;}
    Q_INVOKABLE bool getSendDataStatus(){return m_isSendData;}
    Q_INVOKABLE void switchSendData();
    Q_INVOKABLE void openConfigFile();
    Q_INVOKABLE void getSendedData();

private:
    QString m_node_version{""};
    QSettings m_settings;
    AbstractDiagnostic* m_diagnostic;
    NotifyWorker* m_notifyWorker;

    QTimer *s_uptime_timer;
    QElapsedTimer *s_elapsed_timer;
    QString s_uptime{"00:00:00"};

    QJsonDocument m_lastSendedPack;

    DiagDataSendingWorker *s_diagDataWorker;

    bool m_initStatus{false};
    bool m_isSendData{true};

private slots:
    void slot_diagnostic_data(QJsonDocument);
    void slot_uptime();

private:
    void write_data(QJsonDocument);

signals:
    Q_INVOKABLE void sigSendDataStatusChanged(bool status);
    Q_INVOKABLE void sigInitStatusChanged(bool status);

};

#endif // DIAGNOSTICWORKER_H
