#ifndef NOTIFYWORKER_H
#define NOTIFYWORKER_H

#include <QThread>
#include <QIODevice>
#include <QLocalSocket>
#include <QTcpSocket>
#include <QTimer>
#include <QJsonObject>
#include <QJsonArray>
#include <QJsonDocument>
#include <QDebug>
#include <QProcess>
#include <QVariant>


class NotifyWorker : public QObject
{
    Q_OBJECT
public:
    explicit NotifyWorker(QObject *parent = nullptr);
    ~NotifyWorker();

    bool initWatcher();

    const QString& getSocketState() const {return m_socketState;}

    QVariant readConfig(QString filename, QString section, QString param, QVariant defaultValue);

    bool m_statusInitWatcher{false};
    QJsonDocument m_docNodeInit;
    QJsonArray m_arrNodeInit;
    QStringList m_netList;

    void nodeIsOnline();
    void rcvNetObj(QJsonObject objNet);

public slots:
    void slotError();
    void socketConnected();
    void slotReconnect();
    void socketDisconnected();

    void socketReadyRead();

    void socketStateChanged(QAbstractSocket::SocketState socketState);

signals:
    void changeConnectState(QString);

private:
    void reconnectFunc();
    QByteArrayList jsonListFromData(QByteArray data);

    void procNotifyData(const QJsonObject &rcvObj);
private:
    QString m_configWorkerPath;
    QIODevice *m_socket;
    QString m_listenPath;
    QString m_listenAddr;
    uint16_t m_listenPort;
    QTimer * m_reconnectTimer;
    QTimer * m_initTimer;

    bool m_rcvInitProgress{false};

    QString m_socketState{""};
};

#endif // NOTIFYWORKER_H
