#ifndef NODETRAYCOMMANDCONTROLLER_H
#define NODETRAYCOMMANDCONTROLLER_H

#include <QObject>
#include <QApplication>
#include <QQmlApplicationEngine>
#include <QQmlContext>
#include <QDebug>
#include <QProcess>
#include <QDir>
#include <QDesktopServices>
#include <QUrl>
#include <QMessageBox>

#include <QQmlEngine>

#include "DiagnosticWorker.h"

#ifdef WIN32
#include "registry.h"
#endif

class NodeTrayCommandController : public QObject
{
    Q_OBJECT
    explicit NodeTrayCommandController(QObject *parent = nullptr);

public:
    static NodeTrayCommandController &getInstance();

    void init();
    bool initConfTool();

    QQmlApplicationEngine *qmlEngine();

    enum TypeServiceCommands{
        Status = 0,
        EnableAutostart = 1,
        DisableAutostart,
        Stop,
        Start,
        Restart
    };Q_ENUM(TypeServiceCommands)

    Q_INVOKABLE void runCli();
    Q_INVOKABLE void runTool();
    Q_INVOKABLE void openDir(const QString &dirName);
    Q_INVOKABLE void serviceCommand(TypeServiceCommands type);

    void showMessage(const QString msg = "Could not find cellframe-node-config",
                     const QMessageBox::Icon icn = QMessageBox::Warning);

    DiagnosticWorker &diagnosticWorker = DiagnosticWorker::getInstance();

public:
    bool m_statusInitConftool{false};
    QString m_confToolPath{""};
    QString m_configDir{""};

    QQmlApplicationEngine m_engine;

private:
    QString getConfigPath();
    QString sendRequest(QString req);

    void initEngine();

    QString getResult(QString find, QStringList list);

#ifdef WIN32
    LONG GetDWORDRegKey(HKEY hKey, const std::wstring &strValueName, DWORD &nValue, DWORD nDefaultValue);
    LONG GetBoolRegKey(HKEY hKey, const std::wstring &strValueName, bool &bValue, bool bDefaultValue);
    LONG GetStringRegKey(HKEY hKey, const std::wstring &strValueName, std::wstring &strValue, const std::wstring &strDefaultValue);
#endif

signals:
    Q_INVOKABLE void sigResultStatus(bool status);
    Q_INVOKABLE void sigShowMessage(QString title, QString message);
};

#endif // NODETRAYCOMMANDCONTROLLER_H
