import QtQuick 2.5
import QtQuick.Controls 2.5
import Qt.labs.platform 1.1
import QtQuick.Window 2.3
import QtQuick.Dialogs 1.1

ApplicationWindow {
    id: systemTrayApplicationWindow
    visible: false

    property bool diagStatusInit:false
    property bool diagStatusSendData:true

    Menu {
        id: menu

//        MenuItem {
//            text: qsTr("Run CLI")
//            onTriggered: trayCommandController.runCli()
//        }

//        MenuItem {
//            text: qsTr("Run TOOL")
//            onTriggered: trayCommandController.runTool()
//        }

        Menu{
            title: qsTr("Open dir")

            MenuItem {
                text: qsTr("Configs")
                    onTriggered: trayCommandController.openDir("/etc")
            }
            MenuItem {
                text: qsTr("Wallets")
                    onTriggered: trayCommandController.openDir("/var/lib/wallet")
            }
            MenuItem {
                text: qsTr("Logs")
                    onTriggered: trayCommandController.openDir("/var/log/")
            }
        }

        Menu{
            title: qsTr("Service")
            MenuItem {
                property bool status: false
                id: autostartItemMenu
                text: status ? qsTr("Autostart cellframe-node: enabled"):
                               qsTr("Autostart cellframe-node: disabled")

                onTriggered: {
                    if(status)
                        trayCommandController.serviceCommand(2)
                    else
                        trayCommandController.serviceCommand(1)
                }
            }
            MenuItem {
                visible: CURRENT_OS === "macos" ? false : autostartItemMenu.status
                text: qsTr("Stop cellframe-node")
                onTriggered: trayCommandController.serviceCommand(3)
            }
            MenuItem {
                visible: CURRENT_OS === "macos" ? false : autostartItemMenu.status
                text: qsTr("Start cellframe-node")
                onTriggered: trayCommandController.serviceCommand(4)
            }
            MenuItem {
                visible: CURRENT_OS === "macos" ? false : autostartItemMenu.status
                text: qsTr("Restart cellframe-node")
                onTriggered: trayCommandController.serviceCommand(5)
            }
        }

        Menu{
            title: qsTr("Diagnostic")
            visible: diagStatusInit

            MenuItem {

                text: diagStatusSendData? qsTr("Send diagnostic data: enabled")
                                        : qsTr("Send diagnostic data: disabled")
                onTriggered: diagnosticWorker.switchSendData()
            }
            MenuItem {
                text: qsTr("Sended data list")
                onTriggered: diagnosticWorker.getSendedData()
            }
            MenuItem {
                text: qsTr("Open diagnostic config")
                onTriggered: diagnosticWorker.openConfigFile()
            }

        }
    }

    SystemTrayIcon {
        visible: true
        id: systemTrayIcon

        icon.mask: true
        icon.source: "qrc:/cellframe-node-logo.svg"

        menu: menu

        onActivated:
        {
            trayCommandController.serviceCommand(0)
        }
        Component.onCompleted:
        {
            diagStatusInit = diagnosticWorker.getInitStatus();
            trayCommandController.serviceCommand(0)
        }
    }

    Connections{
        target: trayCommandController
        function onSigResultStatus(status)
        {
//            console.log(status)
            autostartItemMenu.status = status
        }

        function onSigShowMessage(title, message)
        {
            systemTrayIcon.showMessage(title, message, "qrc:/cellframe-node-logo.svg", 5000)
        }
    }

    Connections{
        target: diagnosticWorker
        function onSigInitStatusChanged(status)
        {
            diagStatusInit = status
        }
        function onSigSendDataStatusChanged(status)
        {
            diagStatusSendData = status
        }
    }
}

