#include <QApplication>
#include <QQmlApplicationEngine>
#include <QDebug>

#include <QSystemSemaphore>
#include <QSharedMemory>
#include <QScreen>
#include <sys/stat.h>
#include <memory>

#include "NodeTrayCommandController.h"
#include "DiagnosticWorker.h"


bool SingleApplicationTest(const QString &appName)
{
    static QSystemSemaphore semaphore("<"+appName+" uniq semaphore id>", 1);
    semaphore.acquire();

#ifndef Q_OS_WIN32
    QSharedMemory nix_fix_shared_memory("<"+appName+" uniq memory id>");
    if(nix_fix_shared_memory.attach())
    {
        nix_fix_shared_memory.detach();
    }
#endif

    static QSharedMemory sharedMemory("<"+appName+" uniq memory id>");
    bool is_running;
    if (sharedMemory.attach())
    {
        is_running = true;
    }
    else
    {
        sharedMemory.create(1);
        is_running = false;
    }

    semaphore.release();

    if(is_running)
    {
        QMessageBox msgBox;
        msgBox.setIcon(QMessageBox::Warning);
        msgBox.setText(QObject::tr("The application '%1' is already running.").arg(appName));
        msgBox.exec();
        return false;
    }

    return true;
}

bool findArg(char** begin, char** end, const std::string& option)
{
    return std::find(begin, end, option) != end;
}

QString getDesctopSession()
{
    return qgetenv("DESKTOP_SESSION");
}

int main(int argc, char *argv[])
{

    qSetMessagePattern(QLatin1String(
        "[%{time h:mm:ss}] "
        "%{if-debug}\x1b[36m[DBG]\x1b[37m%{endif}"
        "%{if-info}\x1b[32m[INF]\x1b[37m%{endif}"
        "%{if-warning}\x1b[35m[WRN]\x1b[37m%{endif}"
        "%{if-critical}\x1b[31m[CRT]\x1b[37m%{endif}"
        "%{if-fatal}\x1b[1;31m[FATAL]\x1b[37m%{endif}"
      //"%{appname} %{pid} %{threadid} "
      //"%{file}:%{line} %{function} "
        " %{if-category}%{category} %{endif}%{message}"
        "\x1b[0m"
    ));

#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
    QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
#endif
    
    bool start_tray = findArg(argv, argv+argc,"--tray");

    QCoreApplication *app = start_tray?(new QApplication(argc, argv)):(new QCoreApplication(argc, argv)); 
    if(!start_tray) DiagnosticWorker::getInstance().init();
    else
    {
        QString appName = "cellframe-node-tray";

        if (!SingleApplicationTest(appName))
            return 1;

        QCoreApplication::setApplicationName(appName);

        NodeTrayCommandController::getInstance().init();
    }

    qDebug() << "Desctop session" << getDesctopSession();

    return app->exec();
}
